/**
 * Main program to actually run the Lua code against the C++ object
 */
#include "object.h"
#include "luaobject.h"

// Define the Lua ClassName
const char LuaGameObject::className[] = "LuaGameObject";

// Define the methods we will expose to Lua
// Check luaobject.h for the definitions...
#define method(class, name) {#name, &class::name}
Luna<LuaGameObject>::RegType LuaGameObject::methods[] = {
   method(LuaGameObject, setBalance),
   method(LuaGameObject, getBalance),
   method(LuaGameObject, getMessage),
   method(LuaGameObject, setMessage),
   {0,0}
};

int main(int argc, char *argv[]){
  // Init Lua
  //equivalent to ILOG Rule context creation 
  lua_State *L = lua_open();
  //luaopen_base(L);
  //luaopen_table(L);
  //luaopen_io(L);
  //luaopen_string(L);
  //luaopen_math(L);
  //luaopen_debug(L);
  luaL_openlibs(L);
  //rule context creation ends here
  
  // Register the LuaGameObject data type with Lua
  Luna<LuaGameObject>::Register(L);

  // In C++ - Create a GameObject for use in our program
  GameObject temp(20);
  temp.setMessage("I'm set in C++");
  
  // Push a pointer to this GameObject to the Lua stack
  lua_pushlightuserdata(L, (void*)&temp);//ASSERT
  // And set the global name of this pointer
  lua_setglobal(L,"gameobject");//ASSERT

  printf("In C: %p => %d, %s\n", &temp, temp.getAttr(), temp.getMessage());

  printf("Loading lua----------\n");
  luaL_loadfile(L, argv[1]);//LOAD THE RULE FILE
  printf("lua is loaded--------\n");

  printf("Running lua----------\n");
  lua_pcall(L, 0, 0, 0);//FIREALLRULES
  printf("Lua is done----------\n");
  //lua_setgcthreshold(L, 0);  // collected garbage

  luaL_loadfile(L, argv[1]);
  printf("Running lua2---------\n");
  lua_pcall(L, 0, 0, 0);
  printf("Lua is done2---------\n");

  // GC + Close out Lua
  lua_close(L);

  printf("In C++: %p => %d, %s\n", &temp, temp.getAttr(), temp.getMessage());
  return 0;
}
